clear; %clear workspace

%parameters to test
delta_f=0.2;
phase_step=0;%10/360*2*pi;
start_time=0;

%PLL design constants:
fs=100; %sample rate
Ts=1/fs; %sample time
dds_wordlength=32;
kd=1; %phase detector gain ideal PD
k0_dds=(2*pi)/(2^dds_wordlength)/Ts; %dds gain
k0=k0_dds; 

D_FF=1;
D_FB=1;

Ni=1;
No=1;

%initial condition
f_ref=1.0;
f_dds=f_ref;
initial_phase=0;

%simulation settings
sim_end_time=2;
sim_sample_time=Ts;%/5;
points_per_second=1/sim_sample_time;
N=sim_end_time*points_per_second; %number of samplepoints
sample_time=sim_end_time/N;

%PLL design parameters:
zeta=0.707;
omega_n=2*pi;

k1=(2*zeta*omega_n*No)/(k0*kd);
k2=(omega_n^2*No)/(k0*kd);

% b0=round(k1+k2*Ts);
% b1=round(-k1);
b0=(k1+k2*Ts);
b1=(-k1);

disp('controller parameter:');
disp(strcat('k1 (proportional) = ',num2str(k1, '%10.3e\n')));
disp(strcat('k2 (integral) = ',num2str(k2, '%10.3e\n')));
disp(strcat('b0 = ',num2str(b0, '%10.3e\n')));
disp(strcat('b1 = ',num2str(b1, '%10.3e\n')));
disp(strcat('b0/2^dds_wordlength = ',num2str(b0/2^dds_wordlength, '%10.3e\n')));
disp(strcat('b1/2^dds_wordlength = ',num2str(b1/2^dds_wordlength, '%10.3e\n')));
disp(strcat('b0 (integer) = ',num2str(round(b0), '%d\n')));
disp(strcat('b1 (integer) = ',num2str(round(b1), '%d\n')));


sim_opts = simset('FixedStep',sample_time);
disc_time = 0:Ts:sim_end_time;

%simulation of phase-model
%sim('phasedomain_phase_model1',sim_end_time,sim_opts); %[t,x,y] = sim(model, timespan, options, ut);

%cont. phase model:
time = 0:sim_end_time/(N-1):sim_end_time;
if delta_f > 0
    u_end=delta_f*2*pi*sim_end_time;
    u = 0:u_end/(N-1):u_end;
    error_tf2 = tf([0 0 (1/Ni)*delta_f*2*pi],[1 2*zeta*omega_n omega_n^2]); %1. arg z�hler,2. arg nenner
    [cont_phase_error,cont_time] = impulse(error_tf2,time);
else
    error_tf = tf([(1/Ni) 0 0],[1 2*zeta*omega_n omega_n^2]); %1. arg z�hler,2. arg nenner
    [cont_phase_error,cont_time] = step(error_tf,time);
    cont_phase_error = cont_phase_error * phase_step;
end

%disc. phase model:
if delta_f > 0
    disc_phase_model_error=0; %not implemented now
else
    disc_error_tf = filt([1/Ni -2/Ni 1/Ni],[(1+k0*Ts*kd/No*(k1+k2*Ts)) -(2+k0*Ts*kd*k1/No) 1], Ts);
    [disc_phase_model_error,disc_time] = step(disc_error_tf,disc_time);
    disc_phase_model_error = disc_phase_model_error * phase_step;
end

%simulation of amplitude vs time
sim('discrete_ideal_pll1',sim_end_time,sim_opts); %[t,x,y] = sim(model, timespan, options, ut);

if start_time > 0 
    phase_error_after_freq_step = disc_phase_error(floor(start_time/sim_sample_time):length(disc_phase_error));
else
    phase_error_after_freq_step = disc_phase_error;
end;

if delta_f > 0
    title_phase_error='Frequenzsprungantwort';
else
    title_phase_error='Phasensprungantwort';
end

figure1 = figure(1);
clf reset
axes1 = axes('Parent',figure1);
hold(axes1,'all');
%plot(sim_time, disc_phase_error)
disc_phase_error = disc_phase_error * 180/pi; %norm. to -180...180 degree
cont_phase_error = cont_phase_error * 180/pi;
disc_phase_model_error = disc_phase_model_error * 180/pi;
plot(disc_time, disc_phase_error,'--b','Parent',axes1)
%stem(disc_time, disc_phase_error,'--b','Parent',axes1)
if delta_f == 0
plot(disc_time, disc_phase_model_error,'-.g','Parent',axes1)
end;
plot(cont_time, cont_phase_error,'-.r','Parent',axes1)
if min(disc_phase_error)<max(disc_phase_error)
    axis(axes1,[0 sim_end_time min(disc_phase_error) max(disc_phase_error)]);
end;
title(strcat(title_phase_error,' - Phasenfehler'));
ylabel('\Delta\phi(t) []')
xlabel('t [s]')
legend({'Zeitdiskretes Amplitudenmodell','Zeitkontinuierliches Phasenmodell','Zeitdiskretes Phasenmodell'});
grid

figure2 = figure(2);
clf reset
axes2 = axes('Parent',figure2);
hold(axes2,'all');
plot(disc_time, ftw_correction/2^dds_wordlength*fs)
%stem(disc_time, ftw_correction/2^dds_wordlength/Ts)
plot(disc_time,ones(length(disc_time),1)*delta_f,'-.r')
title('DDS Frequenzkorrektur');
xlabel('t [s]')
ylabel('f_{DDS} [Hz]')
grid

